﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Transactions;
using System.Drawing;
using System.IO;
using System.ComponentModel;

/// <summary>
/// エントリの取得・追加・削除・更新処理を行うビジネスロジッククラスです
/// </summary>
[DataObjectAttribute]
public class EntryBL
{
    /// <summary>
    /// コンストラクタ
    /// </summary>
	public EntryBL()
	{
	}

    /// <summary>
    /// カテゴリ毎のエントリ数を取得します
    /// </summary>
    /// <returns>カテゴリ毎のエントリ数データ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryCountPerCategoryDataTable GetEntryCountPerCategory()
    {
        EntryDs.EntryCountPerCategoryDataTable result = null;

        using (EntryDsTableAdapters.EntryCountPerCategoryTableAdapter adapter = new EntryDsTableAdapters.EntryCountPerCategoryTableAdapter())
        {
            result = adapter.GetData();
            using (CategoryDsTableAdapters.CategoryTableAdapter categoryAdapter = new CategoryDsTableAdapters.CategoryTableAdapter())
            {
                CategoryDs.CategoryDataTable categoryTable =  categoryAdapter.GetData();
                foreach (CategoryDs.CategoryRow categoryRow in categoryTable)
                {
                    EntryDs.EntryCountPerCategoryRow[] targetRow = result.Select("Category=" + categoryRow.ID) as EntryDs.EntryCountPerCategoryRow[];
                    if (0 == targetRow.Length)
                    {
                        EntryDs.EntryCountPerCategoryRow newRow = result.NewEntryCountPerCategoryRow();
                        newRow.Category = categoryRow.ID;
                        newRow.CategoryName = categoryRow.Text;
                        newRow.EntryCount = 0;
                        result.Rows.Add(newRow);
                    }
                    else
                    {
                        targetRow[0].CategoryName = categoryRow.Text;
                    }
                }
                EntryDs.EntryCountPerCategoryRow[] nonCategoryRow = result.Select("Category=-1") as EntryDs.EntryCountPerCategoryRow[];
                if (0 != nonCategoryRow.Length)
                    nonCategoryRow[0].CategoryName = "カテゴリなし";
            }
        }

        return result;
    }

    /// <summary>
    /// 最近追加されたエントリ情報を取得します
    /// </summary>
    /// <returns>エントリデータ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryDataTable GetRecentEntry()
    {
        BlogConfigurationDs.BlogConfigurationDataTable configTable = null;
        EntryDs.EntryDataTable entryTable = null;
        using (BlogConfigurationDsTableAdapters.BlogConfigurationTableAdapter configAdapter = new BlogConfigurationDsTableAdapters.BlogConfigurationTableAdapter())
        {
            configTable = configAdapter.GetData();
            using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
            {
                entryTable = entryAdapter.GetDataRecent(configTable[0].ViewRecentEntryCount);
            }
        }
        return entryTable;
    }

    /// <summary>
    /// 指定されたカテゴリIDに属するエントリ情報を取得します
    /// </summary>
    /// <param name="categoryID">カテゴリID</param>
    /// <returns>エントリデータ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryDataTable GetEntryByCategoryID(int categoryID)
    {
        EntryDs.EntryDataTable entryTable = null;
        using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
        {
            entryTable = entryAdapter.GetDataByCategoryID( categoryID);
        }
        return entryTable;
    }

    /// <summary>
    /// 指定されたIDのエントリ情報を取得します
    /// </summary>
    /// <param name="entryID">エントリID</param>
    /// <returns>エントリデータ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryDataTable GetEntryByID(int entryID)
    {
        EntryDs.EntryDataTable entryTable = null;
        using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
        {
            entryTable = entryAdapter.GetDataByID(entryID);
        }
        return entryTable;
    }

    /// <summary>
    /// 指定した年月のエントリ情報を取得します
    /// </summary>
    /// <param name="yearMonth">年月を表すDateTimeオブジェクト</param>
    /// <returns>エントリデータ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryDataTable GetEntryByDate(string yearMonth)
    {
        EntryDs.EntryDataTable entryTable = null;
        if(6 == yearMonth.Length)
        {
            int year = int.Parse(yearMonth.Substring(0, 4));
            int month = int.Parse(yearMonth.Substring(4, 2));
            using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
            {
                entryTable = entryAdapter.GetDataByDate(year, month);
            }
        }
        return entryTable;
    }

    /// <summary>
    /// 新規のエントリを追加します（位置情報なし）
    /// </summary>
    /// <param name="Title">タイトル</param>
    /// <param name="Contents">内容</param>
    /// <param name="Category">カテゴリID</param>
    /// <param name="Image1FileName">画像ファイル１</param>
    /// <param name="Image2FileName">画像ファイル２</param>
    /// <param name="Image3FileName">画像ファイル３</param>
    /// <param name="EntryDate">エントリ日（年月日時分秒）</param>
    /// <returns>追加されたレコード数（正常終了は1）</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Insert)]
    public int InsertEntry(string Title, string Contents, int Category, string Image1FileName, string Image2FileName, string Image3FileName, DateTime EntryDate)
    {
        int NewIdentity = -1;
        using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
        {
            DateTime entryYearMonth = new DateTime(EntryDate.Year, EntryDate.Month, 1);
            object oIdentity = entryAdapter.InsertEntry(Title, Contents, Category, Image1FileName, Image2FileName, Image3FileName, false, null, null, null, null, null,
                entryYearMonth, EntryDate, DateTime.Now, DateTime.Now);
            NewIdentity = int.Parse(oIdentity.ToString());
        }
        return NewIdentity;
    }

    /// <summary>
    /// 新規のエントリを追加します（位置情報あり）
    /// </summary>
    /// <param name="Title">タイトル</param>
    /// <param name="Contents">内容</param>
    /// <param name="Category">カテゴリID</param>
    /// <param name="Image1FileName">画像ファイル１</param>
    /// <param name="Image2FileName">画像ファイル２</param>
    /// <param name="Image3FileName">画像ファイル３</param>
    /// <param name="EntryDate">エントリ日（年月日時分秒）</param>
    /// <param name="Location_Lng">位置情報の経度</param>
    /// <param name="Location_Lat">位置情報の緯度</param>
    /// <param name="Zoom">位置情報のズーム値</param>
    /// <param name="Marker_Lng">位置情報のマークの経度</param>
    /// <param name="Marker_Lat">位置情報のマークの緯度</param>
    /// <returns>追加されたレコード数（正常終了は1）</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Insert)]
    public int InsertEntryWithLocation(string Title, string Contents, int Category, string Image1FileName, string Image2FileName, string Image3FileName, DateTime EntryDate, float Location_Lng, float Location_Lat, short Zoom, float Marker_Lng, float Marker_Lat)
    {
        int NewIdentity = -1;
        using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
        {
            DateTime entryYearMonth = new DateTime(EntryDate.Year, EntryDate.Month, 1);
            object oIdentity = entryAdapter.InsertEntry(Title, Contents, Category, Image1FileName, Image2FileName, Image3FileName, true, Location_Lng,
                Location_Lat, Zoom, Marker_Lng, Marker_Lat, entryYearMonth, EntryDate, DateTime.Now, DateTime.Now);
            NewIdentity = int.Parse(oIdentity.ToString());
        }
        return NewIdentity;
    }

    /// <summary>
    /// 年月毎のエントリ数情報を取得します
    /// </summary>
    /// <returns>年月毎のエントリ数データ</returns>
    [DataObjectMethodAttribute(DataObjectMethodType.Select)]
    public EntryDs.EntryCountPerMonthDataTable GetEntryCountPerMonth()
    {
        EntryDs.EntryCountPerMonthDataTable result = null;
        using (EntryDsTableAdapters.EntryCountPerMonthTableAdapter adapter = new EntryDsTableAdapters.EntryCountPerMonthTableAdapter())
        {
            result = adapter.GetData();
        }
        return result;
    }

    /// <summary>
    /// 指定したIDのエントリを削除します
    /// </summary>
    /// <param name="ID">エントリID</param>
    [DataObjectMethodAttribute(DataObjectMethodType.Delete)]
    public void DeleteEntry(int ID)
    {
        using (EntryDsTableAdapters.EntryTableAdapter entryAdapter = new EntryDsTableAdapters.EntryTableAdapter())
        {
            entryAdapter.DeleteEntry(ID);
            DeleteImageFiles(ID);
        }
    }

    /// <summary>
    /// 指定したIDのエントリに付加された画像ファイルを削除します
    /// </summary>
    /// <param name="ID">エントリID</param>
    public void DeleteImageFiles(int ID)
    {
        string mapPath = HttpContext.Current.Server.MapPath("~/Image/" + ID.ToString());
        if (Directory.Exists(mapPath))
        {
            Directory.Delete(mapPath, true);
        }
    }

    /// <summary>
    /// 指定されたエントリIDに付加された画像データをファイルシステムに保存します
    /// </summary>
    /// <param name="entryID">エントリID</param>
    /// <param name="fileUploads">画像情報を所有したFileUploadサーバーコントロール配列</param>
    public void SaveImageFiles(int entryID, FileUpload[] fileUploads)
    {
        foreach (FileUpload o in fileUploads)
        {
            if (o.HasFile)
            {
                string mapPath = HttpContext.Current.Server.MapPath("~/Image/" + entryID.ToString());

                string fullPath = Path.Combine(mapPath, o.FileName);
                Directory.CreateDirectory(mapPath);
                o.SaveAs(fullPath);
                string fullPath_small = Path.Combine(mapPath, Path.GetFileNameWithoutExtension(fullPath) + "_small" + Path.GetExtension(fullPath));
                System.Drawing.Image originalImage = System.Drawing.Image.FromFile(fullPath);
                int smallWidth, smallHeight;
                GetSmallImageSize(originalImage.Width, originalImage.Height, out smallWidth, out smallHeight);
                System.Drawing.Image smallImage = originalImage.GetThumbnailImage(smallWidth, smallHeight, null, IntPtr.Zero);
                smallImage.Save(fullPath_small);
            }
        }
    }

    /// <summary>
    /// 画像データを縦横比を維持したままで、小さいサイズ（最大160*120）に縮小した場合の画像サイズを取得します
    /// </summary>
    /// <param name="originalWidth">オリジナルの幅</param>
    /// <param name="originalHeight">オリジナルの高さ</param>
    /// <param name="smallWidth">縮小した場合の幅（出力パラメータ）</param>
    /// <param name="smallHeight">縮小した場合の高さ（出力パラメータ）</param>
    private void GetSmallImageSize(int originalWidth, int originalHeight, out int smallWidth, out int smallHeight)
    {
        if (originalWidth <= 160 && originalHeight <= 120)
        {
            smallWidth = originalWidth;
            smallHeight = originalHeight;
        }
        else
        {
            double widthScale = (double)originalWidth / 160.0;
            double heightScale = (double)originalHeight / 120.0;
            if (widthScale > heightScale)
            {
                smallWidth = 160;
                smallHeight = (int)(originalHeight / widthScale);
            }
            else
            {
                smallWidth = (int)(originalWidth / heightScale);
                smallHeight = 120;
            }
        }
    }
}
